Class: ReplCode
- Inherits:
-
Object
- Object
- ReplCode
- Defined in:
- lib/puppet-repl/repl_code.rb,
lib/puppet-repl/code/loc.rb,
lib/puppet-repl/code/code_range.rb
Overview
Pry::Code is a class that encapsulates lines of source code and their
line numbers and formats them for terminal output. It can read from a file
or method definition or be instantiated with a String or an Array.
In general, the formatting methods in Code return a new Code object
which will format the text as specified when #to_s is called. This allows
arbitrary chaining of formatting methods without mutating the original
object.
Defined Under Namespace
Instance Attribute Summary collapse
-
#code_type ⇒ Symbol
The type of code stored in this wrapper.
Class Method Summary collapse
-
.expression_at(raw, line_number, consume = 0) ⇒ String
Get the multiline expression that starts on the given line number.
-
.from_file(filename, code_type = nil) ⇒ Code
Instantiate a
Codeobject containing code loaded from a file or Pry's line buffer.
Instance Method Summary collapse
-
#==(other) ⇒ Boolean
Two
Codeobjects are equal if they contain the same lines with the same numbers. -
#after(lineno, lines = 1) ⇒ Code
Remove all lines except for the
linesafter and excludinglineno. -
#around(lineno, lines = 1) ⇒ Code
Remove all lines except for the
lineson either side of and includinglineno. -
#before(lineno, lines = 1) ⇒ Code
Remove all lines except for the
linesup to and excludinglineno. -
#between(start_line, end_line = nil) ⇒ Code
Remove all lines that aren't in the given range, expressed either as a
Rangeobject or a first and last line number (inclusive). -
#comment_describing(line_number) ⇒ String
Get the comment that describes the expression on the given line number.
-
#expression_at(line_number, consume = 0) ⇒ String
Get the multiline expression that starts on the given line number.
-
#grep(pattern) ⇒ Code
Remove all lines that don't match the given
pattern. -
#highlighted ⇒ String
A (possibly highlighted) copy of the source code.
-
#initialize(lines = [], start_line = 1, code_type = :ruby) ⇒ ReplCode
constructor
Instantiate a
Codeobject containing code from the givenArray,String, orIO. - #inspect ⇒ String
-
#length ⇒ Integer
Return the number of lines stored.
-
#max_lineno_width ⇒ Integer
The number of digits in the last line.
-
#method_missing(name, *args, &block) ⇒ Object
Forward any missing methods to the output of
#to_s. -
#nesting_at(line_number, top_module = Object) ⇒ Array<Module>
Get the (approximate) Module.nesting at the give line number.
-
#print_to_output(output, color = false) ⇒ Object
Writes a formatted representation (based on the configuration of the object) to the given output, which must respond to
#<<. -
#push(line, lineno = nil) ⇒ String
(also: #<<)
Append the given line.
-
#raw ⇒ String
Return an unformatted String of the code.
-
#select {|LOC| ... } ⇒ Code
Filter the lines using the given block.
-
#take_lines(start_line, num_lines) ⇒ Code
Take
num_linesfromstart_line, forward or backwards. -
#to_s ⇒ String
A formatted representation (based on the configuration of the object).
-
#with_indentation(spaces = 0) ⇒ Code
Format output with the specified number of spaces in front of every line, unless
spacesis falsy. -
#with_line_numbers(y_n = true) ⇒ Code
Format output with line numbers next to it, unless
y_nis falsy. -
#with_marker(lineno = 1) ⇒ Code
Format output with a marker next to the given
lineno, unlesslinenois falsy.
Constructor Details
#initialize(lines = [], start_line = 1, code_type = :ruby) ⇒ ReplCode
Instantiate a Code object containing code from the given Array,
String, or IO. The first line will be line 1 unless specified
otherwise. If you need non-contiguous line numbers, you can create an
empty Code object and then use #push to insert the lines.
44 45 46 47 48 49 50 51 52 53 |
# File 'lib/puppet-repl/repl_code.rb', line 44 def initialize(lines = [], start_line = 1, code_type = :ruby) if lines.is_a? String lines = lines.lines end @lines = lines.each_with_index.map { |line, lineno| LOC.new(line, lineno + start_line.to_i) } @code_type = code_type @with_marker = @with_indentation = nil end |
Dynamic Method Handling
This class handles dynamic methods through the method_missing method
#method_missing(name, *args, &block) ⇒ Object
Forward any missing methods to the output of #to_s.
300 301 302 |
# File 'lib/puppet-repl/repl_code.rb', line 300 def method_missing(name, *args, &block) to_s.send(name, *args, &block) end |
Instance Attribute Details
#code_type ⇒ Symbol
Returns The type of code stored in this wrapper.
34 35 36 |
# File 'lib/puppet-repl/repl_code.rb', line 34 def code_type @code_type end |
Class Method Details
.expression_at(raw, line_number, consume = 0) ⇒ String
Get the multiline expression that starts on the given line number.
257 258 259 260 |
# File 'lib/puppet-repl/repl_code.rb', line 257 def self.expression_at(raw, line_number, consume = 0) #self.class.expression_at(raw, line_number, :consume => consume) raw end |
.from_file(filename, code_type = nil) ⇒ Code
Instantiate a Code object containing code loaded from a file or
Pry's line buffer.
23 24 25 26 27 28 29 30 |
# File 'lib/puppet-repl/repl_code.rb', line 23 def from_file(filename, code_type = nil) if filename == :code new(Puppet[:code], 1, code_type) else code_file = CodeFile.new(filename, code_type) new(code_file.code, 1, code_file.code_type) end end |
Instance Method Details
#==(other) ⇒ Boolean
Two Code objects are equal if they contain the same lines with the same
numbers. Otherwise, call to_s and chomp and compare as Strings.
290 291 292 293 294 295 296 297 |
# File 'lib/puppet-repl/repl_code.rb', line 290 def ==(other) if other.is_a?(Code) other_lines = other.instance_variable_get(:@lines) @lines.each_with_index.all? { |loc, i| loc == other_lines[i] } else to_s.chomp == other.to_s.chomp end end |
#after(lineno, lines = 1) ⇒ Code
Remove all lines except for the lines after and excluding lineno.
147 148 149 150 151 152 153 |
# File 'lib/puppet-repl/repl_code.rb', line 147 def after(lineno, lines = 1) return self unless lineno select do |loc| loc.lineno > lineno && loc.lineno <= lineno + lines end end |
#around(lineno, lines = 1) ⇒ Code
Remove all lines except for the lines on either side of and including
lineno.
134 135 136 137 138 139 140 |
# File 'lib/puppet-repl/repl_code.rb', line 134 def around(lineno, lines = 1) return self unless lineno select do |loc| loc.lineno >= lineno - lines && loc.lineno <= lineno + lines end end |
#before(lineno, lines = 1) ⇒ Code
Remove all lines except for the lines up to and excluding lineno.
120 121 122 123 124 125 126 |
# File 'lib/puppet-repl/repl_code.rb', line 120 def before(lineno, lines = 1) return self unless lineno select do |loc| loc.lineno >= lineno - lines && loc.lineno < lineno end end |
#between(start_line, end_line = nil) ⇒ Code
Remove all lines that aren't in the given range, expressed either as a
Range object or a first and last line number (inclusive). Negative
indices count from the end of the array of lines.
87 88 89 90 91 92 93 94 95 |
# File 'lib/puppet-repl/repl_code.rb', line 87 def between(start_line, end_line = nil) return self unless start_line code_range = CodeRange.new(start_line, end_line) alter do @lines = @lines[code_range.indices_range(@lines)] || [] end end |
#comment_describing(line_number) ⇒ String
Get the comment that describes the expression on the given line number.
241 242 243 |
# File 'lib/puppet-repl/repl_code.rb', line 241 def comment_describing(line_number) self.class.comment_describing(raw, line_number) end |
#expression_at(line_number, consume = 0) ⇒ String
Get the multiline expression that starts on the given line number.
249 250 251 |
# File 'lib/puppet-repl/repl_code.rb', line 249 def expression_at(line_number, consume = 0) self.class.expression_at(raw, line_number, :consume => consume) end |
#grep(pattern) ⇒ Code
Remove all lines that don't match the given pattern.
159 160 161 162 163 164 165 166 |
# File 'lib/puppet-repl/repl_code.rb', line 159 def grep(pattern) return self unless pattern pattern = Regexp.new(pattern) select do |loc| loc.line =~ pattern end end |
#highlighted ⇒ String
Returns a (possibly highlighted) copy of the source code.
219 220 221 |
# File 'lib/puppet-repl/repl_code.rb', line 219 def highlighted print_to_output("", true) end |
#inspect ⇒ String
203 204 205 |
# File 'lib/puppet-repl/repl_code.rb', line 203 def inspect Object.instance_method(:to_s).bind(self).call end |
#length ⇒ Integer
Return the number of lines stored.
281 282 283 |
# File 'lib/puppet-repl/repl_code.rb', line 281 def length @lines ? @lines.length : 0 end |
#max_lineno_width ⇒ Integer
Returns the number of digits in the last line.
208 209 210 |
# File 'lib/puppet-repl/repl_code.rb', line 208 def max_lineno_width @lines.length > 0 ? @lines.last.lineno.to_s.length : 0 end |
#nesting_at(line_number, top_module = Object) ⇒ Array<Module>
Get the (approximate) Module.nesting at the give line number.
267 268 269 |
# File 'lib/puppet-repl/repl_code.rb', line 267 def nesting_at(line_number, top_module = Object) Indent.nesting_at(raw, line_number) end |
#print_to_output(output, color = false) ⇒ Object
Writes a formatted representation (based on the configuration of the
object) to the given output, which must respond to #<<.
225 226 227 228 229 230 231 232 233 234 235 |
# File 'lib/puppet-repl/repl_code.rb', line 225 def print_to_output(output, color=false) @lines.each do |loc| loc = loc.dup loc.add_line_number(max_lineno_width) if @with_line_numbers loc.add_marker(@marker_lineno) if @with_marker loc.indent(@indentation_num) if @with_indentation output << loc.line output << "\n" end output end |
#push(line, lineno = nil) ⇒ String Also known as: <<
Append the given line. lineno is one more than the last existing
line, unless specified otherwise.
61 62 63 64 65 66 67 |
# File 'lib/puppet-repl/repl_code.rb', line 61 def push(line, lineno = nil) if lineno.nil? lineno = @lines.last.lineno + 1 end @lines.push(LOC.new(line, lineno)) line end |
#raw ⇒ String
Return an unformatted String of the code.
274 275 276 |
# File 'lib/puppet-repl/repl_code.rb', line 274 def raw @lines.map(&:line).join("\n") << "\n" end |
#select {|LOC| ... } ⇒ Code
Filter the lines using the given block.
74 75 76 77 78 |
# File 'lib/puppet-repl/repl_code.rb', line 74 def select(&block) alter do @lines = @lines.select(&block) end end |
#take_lines(start_line, num_lines) ⇒ Code
Take num_lines from start_line, forward or backwards.
102 103 104 105 106 107 108 109 110 111 112 113 |
# File 'lib/puppet-repl/repl_code.rb', line 102 def take_lines(start_line, num_lines) start_idx = if start_line >= 0 @lines.index { |loc| loc.lineno >= start_line } || @lines.length else [@lines.length + start_line, 0].max end alter do @lines = @lines.slice(start_idx, num_lines) end end |
#to_s ⇒ String
Returns a formatted representation (based on the configuration of the object).
214 215 216 |
# File 'lib/puppet-repl/repl_code.rb', line 214 def to_s print_to_output("", false) end |
#with_indentation(spaces = 0) ⇒ Code
Format output with the specified number of spaces in front of every line,
unless spaces is falsy.
195 196 197 198 199 200 |
# File 'lib/puppet-repl/repl_code.rb', line 195 def with_indentation(spaces = 0) alter do @with_indentation = !!spaces @indentation_num = spaces end end |
#with_line_numbers(y_n = true) ⇒ Code
Format output with line numbers next to it, unless y_n is falsy.
172 173 174 175 176 |
# File 'lib/puppet-repl/repl_code.rb', line 172 def with_line_numbers(y_n = true) alter do @with_line_numbers = y_n end end |
#with_marker(lineno = 1) ⇒ Code
Format output with a marker next to the given lineno, unless lineno is
falsy.
183 184 185 186 187 188 |
# File 'lib/puppet-repl/repl_code.rb', line 183 def with_marker(lineno = 1) alter do @with_marker = !!lineno @marker_lineno = lineno end end |